//
//  NSString_AMAdditions.m
//  MausX
//
//  Created by Andreas on Mon Oct 20 2003.
//  Copyright (c) 2003 Andreas Mayer. All rights reserved.
//
//
//	2009-10-31	Andreas Mayer
//	- fixed a bug where -componentsSeparatedByLineDelimiter would not include the last line
//   (probably a regression with a more recent OS version)


#import "NSString_AMAdditions.h"
#import "AMSDKCompatibility.h"


NSString *AMStringFromRect(NSRect rect)
{
	return [NSString stringWithFormat:@"%f %f %f %f", rect.origin.x, rect.origin.y, rect.size.width, rect.size.height];
}

NSRect AMRectFromString(NSString *string)
{
	NSRect result;
	NSArray *values = [string componentsSeparatedByString:@" "];
	result = NSMakeRect([[values objectAtIndex:0] floatValue], [[values objectAtIndex:1] floatValue], [[values objectAtIndex:2] floatValue], [[values objectAtIndex:3] floatValue]);
	return result;
}


@implementation NSString (AMAdditions)

+ (NSString *)randomStringOfLength:(int)length withCharacters:(NSString *)characters
{
	NSMutableString *result = [[[NSMutableString alloc] init] autorelease];
	NSInteger numberOfCharacters = [characters length];
	NSRange range = NSMakeRange(0, 1);
	// initialite random number generator
	srandom((unsigned)[[NSDate date] timeIntervalSince1970]); //just some semi-random seed
	int i;
	for (i = 0; i<length; i++) {
		range.location = random()%numberOfCharacters;
		[result appendString:[characters substringWithRange:range]];
	}
	return result;
}

+ (NSString *)uniqueString
{
	CFUUIDRef uuid = CFUUIDCreate(NULL);
	CFStringRef uuidStr = CFUUIDCreateString(NULL, uuid);
	CFRelease(uuid);
	[(NSString *)uuidStr autorelease];
	return (NSString *)uuidStr;
}

+ (NSString *)validStringFromString:(NSString *)string
{
	// returns string if string is not nil, @"" otherwise
	return ((string) ? string : @"");
}

- (BOOL)isEmptyString
{
	return [self isEqualToString:@""];
}

- (BOOL)isValidString;
{
	return ![self isEqualToString:@""];
}

- (NSString *)lastCharacter
{
	NSString *result = nil;
	NSInteger len;
	if ((len = [self length]) > 0) {
		result = [self substringWithRange:NSMakeRange(len-1, 1)];
	}
	return result;
}

- (NSString *)initialCharacters
{
	NSMutableString *result = [[[NSMutableString alloc] init] autorelease];
	NSScanner *scanner = [NSScanner scannerWithString:self];
	NSString *buffer = [[[NSString alloc] init] autorelease];
	while (![scanner isAtEnd]) {
		// skip non letter characters
		[scanner scanUpToCharactersFromSet:[NSCharacterSet letterCharacterSet] intoString:nil];
		[scanner scanCharactersFromSet:[NSCharacterSet letterCharacterSet] intoString:&buffer];
		[result appendString:[buffer substringToIndex:1]];
	}
	return result;
}

- (NSArray *)componentsSeparatedByLineDelimiter
{
	NSMutableArray *result = [[[NSMutableArray alloc] init] autorelease];
	NSUInteger start;
	NSUInteger end;
	NSUInteger next;
	NSUInteger stringLength;
	NSRange searchRange;
	NSRange lineRange;
	
	stringLength = [self length];
	searchRange = NSMakeRange(0, 0);
	do {
		[self getLineStart:&start end:&next contentsEnd:&end forRange:searchRange];
		lineRange.location = start;
		if (end > 0) {
			lineRange.length = end-start;
		} else {
			lineRange.length = next-start;
		}
		[result addObject:[self substringWithRange:lineRange]];
		searchRange.location = next;
	} while (next < stringLength);
	return result;
}

- (unsigned long)unsignedLongValue
{
	long long result = 0;
	NSScanner *scanner = [NSScanner scannerWithString:self];
	if ([scanner scanLongLong:&result]) {
		if ((result < 0) || (result > ((unsigned long)LONG_MAX-(unsigned long)LONG_MIN))) {
			result = 0;
		}
	}
	return result;
}

- (NSString *)lastKeyComponent
{
	NSString *result = self;
	NSRange dotRange = [self rangeOfString:@"." options:NSBackwardsSearch];
	if (dotRange.location != NSNotFound) {
		result = [self substringFromIndex:dotRange.location+1];
	}
	return result;
}

- (BOOL)boolValue
{
	return ([[self uppercaseString] isEqualToString:@"YES"] || ![self isEqualToString:@"0"] || [[self uppercaseString] isEqualToString:@"TRUE"]);
}

- (NSString *)stringWithFirst:(unsigned int)number componentsSeparatedByString:(NSString *)separator
{
	return [self stringWithFirst:number componentsSeparatedByString:separator ellipsisString:@""];
}

- (NSString *)stringWithFirst:(unsigned int)number componentsSeparatedByString:(NSString *)separator ellipsisString:(NSString *)ellipsisString;
{
	NSString *result = self;
	NSArray *components = [self componentsSeparatedByString:separator];
	if ([components count] > number) {
		NSRange subrange = NSMakeRange(0, number);
		result = [[components subarrayWithRange:subrange] componentsJoinedByString:separator];
		result = [result stringByAppendingString:ellipsisString];
	}
	return result;
}

- (NSString *)stringByRemovingHTMLUsingDelimiter:(NSString *)delimiter
{
	NSString *result = @"";
	NSString *partialString;
	NSScanner *scanner = [NSScanner scannerWithString:self];
	BOOL found;
	do {
		found = [scanner scanUpToString:@"<" intoString:&partialString];
		//int loc = [scanner scanLocation];
		if ([scanner scanUpToString:@">" intoString:nil]) {
			[scanner scanString:@">" intoString:nil];
		}
		//loc = [scanner scanLocation];
		if (found) {
			if (![result isEmptyString] && delimiter) {
				result = [result stringByAppendingString:delimiter];
			}
			result = [result stringByAppendingString:partialString];
		}
	} while (found);
	return result;
}

- (NSString *)stringWithFirstURLComponent
{
	NSMutableString *result = [[[NSMutableString alloc] init] autorelease];
	NSString *schemePart = @"";
	NSString *hostPart = @"";
	NSScanner *scanner = [NSScanner scannerWithString:self];
	if ([scanner scanUpToString:@"://" intoString:&schemePart]) {
		[scanner scanString:@"://" intoString:nil];
		[scanner scanUpToString:@"/" intoString:&hostPart];
	}
	[result appendString:schemePart];
	[result appendString:@"://"];
	[result appendString:hostPart];
	return [[result copy] autorelease];
}

- (NSString *)stringByEscapingEntities
{
	NSString *result;
	CFStringRef escapedStringRef = CFXMLCreateStringByEscapingEntities(NULL, (CFStringRef)self, NULL);
	result = [NSString stringWithString:(NSString *)escapedStringRef];
	CFRelease(escapedStringRef);
	return result;
}

- (NSString *)stringByEscapingEntities:(NSDictionary *)entities
{
	NSString *result;
	CFStringRef escapedStringRef = CFXMLCreateStringByEscapingEntities(NULL, (CFStringRef)self, (CFDictionaryRef)entities);
	result = [NSString stringWithString:(NSString *)escapedStringRef];
	CFRelease(escapedStringRef);
	return result;
}

- (NSString *)stringByUnescapingEntities
{
	NSString *result;
	CFStringRef unescapedStringRef = CFXMLCreateStringByUnescapingEntities(NULL, (CFStringRef)self, NULL);
	result = [NSString stringWithString:(NSString *)unescapedStringRef];
	CFRelease(unescapedStringRef);
	return result;
}

- (NSString *)stringByUnescapingEntities:(NSDictionary *)entities
{
	NSString *result;
	CFStringRef unescapedStringRef = CFXMLCreateStringByUnescapingEntities(NULL, (CFStringRef)self, (CFDictionaryRef)entities);
	result = [NSString stringWithString:(NSString *)unescapedStringRef];
	CFRelease(unescapedStringRef);
	return result;
}

- (NSString *)stringByTrimmingWhitespace
{
	return [self stringByTrimmingCharactersInSet:[NSCharacterSet whitespaceCharacterSet]];
}

- (NSString *)stringByReplacingCharactersInSet:(NSCharacterSet *)characterSet withString:(NSString *)string
{
	NSMutableString *result = [[[NSMutableString alloc] init] autorelease];
	NSScanner *scanner = [NSScanner scannerWithString:self];
	NSString *buffer;
	NSString *delimiter;
	BOOL found = NO;
	while (![scanner isAtEnd]) {
		buffer = [[[NSString alloc] init] autorelease];
		[scanner scanUpToCharactersFromSet:characterSet intoString:&buffer];
		found = [scanner scanCharactersFromSet:characterSet intoString:&delimiter];
		[result appendString:buffer];
		if (found) {
			[result appendString:string];
		}
	}
	return result;
}

- (NSString *)stringByAppendingEllipsis
{
	return [self stringByAppendingString:[NSString stringWithUTF8String:"\xE2\x80\xA6"]]; 
}

- (NSComparisonResult)numericCompare:(NSString *)string
{
	return [self compare:string options:NSNumericSearch];
}

- (NSData *)hexValue
{
	NSData *result = nil;
	if ([self length] > 0) {
		NSUInteger dataBytes = ([self length]+1)/2;
		char *buffer;
		buffer = malloc(dataBytes);
#if MAC_OS_X_VERSION_10_4 <= MAC_OS_X_VERSION_MAX_ALLOWED
		const char *string = [self cStringUsingEncoding:NSASCIIStringEncoding];
#else
		const char *string = [self cString];
#endif
		int pos = -1;
		int i = 0;
		for (i=0; i < strlen(string); i++) {
			if ((i%2) == 0) {
				pos++;
				buffer[pos] = 0;
			} else {
				buffer[pos] = buffer[pos] << 4;
			}
			if (string[i] >= '0' && string[i] <= '9')	{
				buffer[pos] += string[i] - '0';
			} else if (string[i] >= 'A' && string[i] <= 'F') {
				buffer[pos] += string[i] - 55;
			} else if (string[i] >= 'a' && string[i] <= 'f') {
				buffer[pos] += string[i] - 87;
			}
		}
		result = [NSData dataWithBytes:buffer length:dataBytes];
		free(buffer);
	}
	return result;
}


@end
